@@ -1,14 +1,14 @@
 
-/*
- *  $Id: 894017c5ed9c76c4c797d6d60cb977c575f946d5 $
+/**
+ * @file expGuile.c
  *
- *  Time-stamp:        "2010-02-24 08:43:00 bkorb"
+ *  Time-stamp:        "2012-03-04 19:06:37 bkorb"
  *
  *  This module implements the expression functions that should
  *  be part of Guile.
  *
  *  This file is part of AutoGen.
- *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ *  Copyright (c) 1992-2012 Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -25,93 +25,59 @@
  */
 
 LOCAL teGuileType
-gh_type_e( SCM typ )
+ag_scm_type_e(SCM typ)
 {
-    if (AG_SCM_BOOL_P(   typ ))   return GH_TYPE_BOOLEAN;
-    if (AG_SCM_SYM_P(    typ ))   return GH_TYPE_SYMBOL;
-    if (AG_SCM_STRING_P( typ ))   return GH_TYPE_STRING;
-    if (AG_SCM_IS_PROC(  typ ))   return GH_TYPE_PROCEDURE;
-    if (AG_SCM_CHAR_P(   typ ))   return GH_TYPE_CHAR;
-    if (AG_SCM_VEC_P(    typ ))   return GH_TYPE_VECTOR;
-    if (AG_SCM_PAIR_P(   typ ))   return GH_TYPE_PAIR;
-    if (AG_SCM_NUM_P(    typ ))   return GH_TYPE_NUMBER;
-    if (AG_SCM_LIST_P(   typ ))   return GH_TYPE_LIST;
+    if (AG_SCM_BOOL_P(  typ)) return GH_TYPE_BOOLEAN;
+    if (AG_SCM_SYM_P(   typ)) return GH_TYPE_SYMBOL;
+    if (AG_SCM_STRING_P(typ)) return GH_TYPE_STRING;
+    if (AG_SCM_IS_PROC( typ)) return GH_TYPE_PROCEDURE;
+    if (AG_SCM_CHAR_P(  typ)) return GH_TYPE_CHAR;
+    if (AG_SCM_VEC_P(   typ)) return GH_TYPE_VECTOR;
+    if (AG_SCM_PAIR_P(  typ)) return GH_TYPE_PAIR;
+    if (AG_SCM_NUM_P(   typ)) return GH_TYPE_NUMBER;
+    if (AG_SCM_LIST_P(  typ)) return GH_TYPE_LIST;
 
     return GH_TYPE_UNDEFINED;
 }
 
 
 LOCAL SCM
-ag_scm_c_eval_string_from_file_line( tCC* pzExpr, tCC* pzFile, int line )
+ag_scm_c_eval_string_from_file_line(
+    char const * pzExpr, char const * pzFile, int line)
 {
-    SCM port;
+    SCM port = scm_open_input_string(AG_SCM_STR02SCM(pzExpr));
 
-    if (OPT_VALUE_TRACE >= TRACE_EVERYTHING) {
-        fprintf( pfTrace, "eval from file %s line %d:\n%s\n", pzFile, line,
-                 pzExpr );
-    }
+    if (OPT_VALUE_TRACE >= TRACE_EVERYTHING)
+        fprintf(trace_fp, TRACE_EVAL_STRING, pzFile, line, pzExpr);
 
-#if GUILE_VERSION < 106000
     {
-        tSCC zEx[] = "eval-string-from-file-line";
-        SCM  expr  = scm_makfrom0str( pzExpr );
-        port = scm_mkstrport( SCM_INUM0, expr, SCM_OPN | SCM_RDNG, zEx );
-    }
-#else
-    port = scm_open_input_string( AG_SCM_STR02SCM( pzExpr ));
-#endif
+        static SCM    file      = SCM_UNDEFINED;
+        static char * pzOldFile = NULL;
 
-#if GUILE_VERSION < 107000
-    {
-        static SCM   file = SCM_UNDEFINED;
-        static char* pzFl = NULL;
-
-        scm_t_port* pt;
-
-        if (  (pzFl == NULL)
-           || (strcmp( AG_SCM_CHARS( file ), pzFile ) != 0) )  {
-            if (pzFl != NULL)
-                AGFREE(pzFl);
-            AGDUPSTR(pzFl, pzFile, "eval file name");
-            file = AG_SCM_STR02SCM( pzFile );
-        }
-
-        pt = SCM_PTAB_ENTRY(port);
-        pt->line_number = line - 1;
-        pt->file_name   = file;
-    }
-
-#else
-    {
-        static SCM file = SCM_UNDEFINED;
-        static char* pzOldFile = NULL;
-
-        if ((pzOldFile == NULL) || (strcmp( pzOldFile, pzFile ) != 0)) {
+        if ((pzOldFile == NULL) || (strcmp(pzOldFile, pzFile) != 0)) {
             if (pzOldFile != NULL)
-                AGFREE( pzOldFile );
-
-            AGDUPSTR( pzOldFile, pzFile, "scheme file source" );
-            file = scm_from_locale_string( pzFile );
+                AGFREE(pzOldFile);
+            AGDUPSTR(pzOldFile, pzFile, "scheme source");
+            file = AG_SCM_STR02SCM(pzFile);
         }
 
-        scm_set_port_filename_x( port, file );
+        {
+            SCM ln = AG_SCM_INT2SCM(line);
+            scm_set_port_filename_x(port, file);
+            scm_set_port_line_x(port, ln);
+            scm_set_port_column_x(port, SCM_INUM0);
+        }
     }
 
-    {
-        SCM ln = scm_from_int( line );
-        scm_set_port_line_x( port, ln );
-    }
-#endif
-
     {
         SCM ans = SCM_UNSPECIFIED;
 
         /* Read expressions from that port; ignore the values.  */
         for (;;) {
-            SCM form = scm_read( port );
-            if (SCM_EOF_OBJECT_P( form ))
+            SCM form = scm_read(port);
+            if (SCM_EOF_OBJECT_P(form))
                 break;
-            ans = scm_primitive_eval_x( form );
+            ans = scm_primitive_eval_x(form);
         }
 
         return ans;
@@ -129,7 +95,7 @@ ag_scm_c_eval_string_from_file_line( tCC* pzExpr, tCC* pzFile, int line )
  * doc:  Return the maximum value in the list
 =*/
 SCM
-ag_scm_max( SCM list )
+ag_scm_max(SCM list)
 {
     int   len;
     SCM   car;
@@ -137,17 +103,17 @@ ag_scm_max( SCM list )
 #   ifndef MAX
 #     define MAX(m,v) ((v > m) ? v : m)
 #   endif
-    len = scm_ilength( list );
+    len = scm_ilength(list);
     if (len <= 0)
         return SCM_UNDEFINED;
 
     while (--len >= 0) {
         long val;
 
-        car  = SCM_CAR( list );
-        list = SCM_CDR( list );
+        car  = SCM_CAR(list);
+        list = SCM_CDR(list);
 
-        switch (gh_type_e( car )) {
+        switch (ag_scm_type_e(car)) {
         case GH_TYPE_BOOLEAN:
             if (car == SCM_BOOL_F) {
                 val = 0;
@@ -157,25 +123,24 @@ ag_scm_max( SCM list )
             break;
 
         case GH_TYPE_CHAR:
-            val = (int)gh_scm2char( car );
+            val = (int)AG_SCM_CHAR(car);
             break;
 
         case GH_TYPE_NUMBER:
-            val = gh_scm2long( car );
+            val = AG_SCM_TO_LONG(car);
             break;
 
         case GH_TYPE_STRING:
-            val = strtol( ag_scm2zchars( car, "number-in-string" ),
-                          NULL, 0 );
+            val = strtol(ag_scm2zchars(car, "number-in-string"), NULL, 0);
             break;
 
         default:
             continue;
         }
-        max_val = MAX( max_val, val );
+        max_val = MAX(max_val, val);
     }
 
-    return gh_long2scm( max_val );
+    return AG_SCM_FROM_LONG(max_val);
 }
 
 
@@ -189,7 +154,7 @@ ag_scm_max( SCM list )
  * doc:  Return the minimum value in the list
 =*/
 SCM
-ag_scm_min( SCM list )
+ag_scm_min(SCM list)
 {
     int   len;
     SCM   car;
@@ -197,17 +162,17 @@ ag_scm_min( SCM list )
 #   ifndef MIN
 #     define MIN(m,v) ((v < m) ? v : m)
 #   endif
-    len = scm_ilength( list );
+    len = scm_ilength(list);
     if (len <= 0)
         return SCM_UNDEFINED;
 
     while (--len >= 0) {
         long val;
 
-        car  = SCM_CAR( list );
-        list = SCM_CDR( list );
+        car  = SCM_CAR(list);
+        list = SCM_CDR(list);
 
-        switch (gh_type_e( car )) {
+        switch (ag_scm_type_e(car)) {
         case GH_TYPE_BOOLEAN:
             if (car == SCM_BOOL_F) {
                 val = 0;
@@ -217,25 +182,24 @@ ag_scm_min( SCM list )
             break;
 
         case GH_TYPE_CHAR:
-            val = (int)gh_scm2char( car );
+            val = (int)AG_SCM_CHAR(car);
             break;
 
         case GH_TYPE_NUMBER:
-            val = gh_scm2long( car );
+            val = AG_SCM_TO_LONG(car);
             break;
 
         case GH_TYPE_STRING:
-            val = strtol( ag_scm2zchars( car, "number-in-string" ),
-                          NULL, 0 );
+            val = strtol(ag_scm2zchars(car, "number-in-string"), NULL, 0);
             break;
 
         default:
             continue;
         }
-        min_val = MIN( min_val, val );
+        min_val = MIN(min_val, val);
     }
 
-    return gh_long2scm( min_val );
+    return AG_SCM_FROM_LONG(min_val);
 }
 
 
@@ -249,36 +213,35 @@ ag_scm_min( SCM list )
  * doc:  Compute the sum of the list of expressions.
 =*/
 SCM
-ag_scm_sum( SCM list )
+ag_scm_sum(SCM list)
 {
-    int  len = scm_ilength( list );
+    int  len = scm_ilength(list);
     long sum = 0;
 
     if (len <= 0)
-        return AG_SCM_INT2SCM( 0 );
+        return AG_SCM_INT2SCM(0);
 
     do  {
-        SCM  car = SCM_CAR( list );
-        list = SCM_CDR( list );
-        switch (gh_type_e( car )) {
+        SCM  car = SCM_CAR(list);
+        list = SCM_CDR(list);
+        switch (ag_scm_type_e(car)) {
         default:
             return SCM_UNDEFINED;
 
         case GH_TYPE_CHAR:
-            sum += (long)(unsigned char)gh_scm2char( car );
+            sum += (long)(unsigned char)AG_SCM_CHAR(car);
             break;
 
         case GH_TYPE_NUMBER:
-            sum += gh_scm2long( car );
+            sum += AG_SCM_TO_LONG(car);
             break;
 
         case GH_TYPE_STRING:
-            sum += strtol( ag_scm2zchars( car, "number-in-string" ),
-                           NULL, 0 );
+            sum += strtol(ag_scm2zchars(car, "number-in-string"), NULL, 0);
         }
     } while (--len > 0);
 
-    return gh_long2scm( sum );
+    return AG_SCM_FROM_LONG(sum);
 }
 
 
@@ -294,32 +257,28 @@ ag_scm_sum( SCM list )
  *       character type (i.e. non-graphic and non-white) will cause a failure.
 =*/
 SCM
-ag_scm_string_to_c_name_x( SCM str )
+ag_scm_string_to_c_name_x(SCM str)
 {
-    tSCC  zFun[] = "ag_scm_string_to_c_name_x";
-    tSCC  zMap[] = "cannot map unprintable chars to C name chars";
     int   len;
     char* pz;
 
-    if (! AG_SCM_STRING_P( str ))
-        scm_wrong_type_arg( zFun, 1, str );
+    if (! AG_SCM_STRING_P(str))
+        scm_wrong_type_arg(STR_TO_C_NAME, 1, str);
+
+    
+    
+    for (pz = (char*)(void*)AG_SCM_CHARS(str), len = AG_SCM_STRLEN(str);
+         --len >= 0;
+         pz++) {
 
-    len = AG_SCM_STRLEN( str );
-    pz  = (char*)(void*)AG_SCM_CHARS( str );
-    while (--len >= 0) {
         char ch = *pz;
-        if (! isalnum( ch )) {
+        if (IS_ALPHANUMERIC_CHAR(ch) || IS_WHITESPACE_CHAR(ch))
+            continue;
 
-            if (isspace( ch ))
-                ;
+        if (! IS_GRAPHIC_CHAR(ch))
+            scm_misc_error(STR_TO_C_NAME, STR_TO_C_MAP_FAIL, str);
 
-            else if (isprint( ch ))
-                *pz = '_';
-
-            else
-                scm_misc_error( zFun, zMap, str );
-        }
-        pz++;
+        *pz = '_';
     }
 
     return str;
@@ -336,20 +295,20 @@ ag_scm_string_to_c_name_x( SCM str )
  * doc:  Change to upper case all the characters in an SCM string.
 =*/
 SCM
-ag_scm_string_upcase_x( SCM str )
+ag_scm_string_upcase_x(SCM str)
 {
     int   len;
     char* pz;
 
-    if (! AG_SCM_STRING_P( str ))
+    if (! AG_SCM_STRING_P(str))
         return SCM_UNDEFINED;
 
-    len = AG_SCM_STRLEN( str );
-    pz  = (char*)(void*)AG_SCM_CHARS( str );
+    len = AG_SCM_STRLEN(str);
+    pz  = (char*)(void*)AG_SCM_CHARS(str);
     while (--len >= 0) {
          char ch = *pz;
-        if (islower( ch ))
-            *pz = toupper( ch );
+        if (IS_LOWER_CASE_CHAR(ch))
+            *pz = toupper(ch);
         pz++;
     }
 
@@ -368,14 +327,14 @@ ag_scm_string_upcase_x( SCM str )
  *       only all the lower case letters are changed to upper case.
 =*/
 SCM
-ag_scm_string_upcase( SCM str )
+ag_scm_string_upcase(SCM str)
 {
     SCM res;
-    if (! AG_SCM_STRING_P( str ))
+    if (! AG_SCM_STRING_P(str))
         return SCM_UNDEFINED;
 
-    res = AG_SCM_STR2SCM( AG_SCM_CHARS( str ), AG_SCM_STRLEN( str ));
-    scm_string_upcase_x( res );
+    res = AG_SCM_STR2SCM(AG_SCM_CHARS(str), AG_SCM_STRLEN(str));
+    scm_string_upcase_x(res);
     return res;
 }
 
@@ -390,31 +349,31 @@ ag_scm_string_upcase( SCM str )
  * doc:  capitalize all the words in an SCM string.
 =*/
 SCM
-ag_scm_string_capitalize_x( SCM str )
+ag_scm_string_capitalize_x(SCM str)
 {
     int     len;
     char*   pz;
-    ag_bool word_start = AG_TRUE;
+    bool word_start = true;
 
-    if (! AG_SCM_STRING_P( str ))
+    if (! AG_SCM_STRING_P(str))
         return SCM_UNDEFINED;
 
-    len = AG_SCM_STRLEN( str );
-    pz  = (char*)(void*)AG_SCM_CHARS( str );
+    len = AG_SCM_STRLEN(str);
+    pz  = (char*)(void*)AG_SCM_CHARS(str);
 
     while (--len >= 0) {
         char ch = *pz;
 
-        if (! isalnum( ch )) {
-            word_start = AG_TRUE;
+        if (! IS_ALPHANUMERIC_CHAR(ch)) {
+            word_start = true;
 
         } else if (word_start) {
-            word_start = AG_FALSE;
-            if (islower( ch ))
-                *pz = toupper( ch );
+            word_start = false;
+            if (IS_LOWER_CASE_CHAR(ch))
+                *pz = toupper(ch);
 
-        } else if (isupper( ch ))
-            *pz = tolower( ch );
+        } else if (IS_UPPER_CASE_CHAR(ch))
+            *pz = tolower(ch);
 
         pz++;
     }
@@ -435,14 +394,14 @@ ag_scm_string_capitalize_x( SCM str )
  *       other letters are made lower case.
 =*/
 SCM
-ag_scm_string_capitalize( SCM str )
+ag_scm_string_capitalize(SCM str)
 {
     SCM res;
-    if (! AG_SCM_STRING_P( str ))
+    if (! AG_SCM_STRING_P(str))
         return SCM_UNDEFINED;
 
-    res = AG_SCM_STR2SCM( AG_SCM_CHARS( str ), AG_SCM_STRLEN( str ));
-    ag_scm_string_capitalize_x( res );
+    res = AG_SCM_STR2SCM(AG_SCM_CHARS(str), AG_SCM_STRLEN(str));
+    ag_scm_string_capitalize_x(res);
     return res;
 }
 
@@ -457,20 +416,20 @@ ag_scm_string_capitalize( SCM str )
  * doc:  Change to lower case all the characters in an SCM string.
 =*/
 SCM
-ag_scm_string_downcase_x( SCM str )
+ag_scm_string_downcase_x(SCM str)
 {
     int   len;
     char* pz;
 
-    if (! AG_SCM_STRING_P( str ))
+    if (! AG_SCM_STRING_P(str))
         return SCM_UNDEFINED;
 
-    len = AG_SCM_STRLEN( str );
-    pz  = (char*)(void*)AG_SCM_CHARS( str );
+    len = AG_SCM_STRLEN(str);
+    pz  = (char*)(void*)AG_SCM_CHARS(str);
     while (--len >= 0) {
         char ch = *pz;
-        if (isupper( ch ))
-            *pz = tolower( ch );
+        if (IS_UPPER_CASE_CHAR(ch))
+            *pz = tolower(ch);
         pz++;
     }
 
@@ -489,16 +448,71 @@ ag_scm_string_downcase_x( SCM str )
  *       only all the upper case letters are changed to lower case.
 =*/
 SCM
-ag_scm_string_downcase( SCM str )
+ag_scm_string_downcase(SCM str)
 {
     SCM res;
-    if (! AG_SCM_STRING_P( str ))
+    if (! AG_SCM_STRING_P(str))
         return SCM_UNDEFINED;
 
-    res = AG_SCM_STR2SCM( AG_SCM_CHARS( str ), AG_SCM_STRLEN( str ));
-    ag_scm_string_downcase_x( res );
+    res = AG_SCM_STR2SCM(AG_SCM_CHARS(str), AG_SCM_STRLEN(str));
+    ag_scm_string_downcase_x(res);
     return res;
 }
+
+
+/*=gfunc string_to_camelcase
+ *
+ * what:   make a string be CamelCase
+ * general_use:
+ *
+ * exparg: str , input/output string
+ *
+ * doc:  Capitalize the first letter of each block of letters and numbers,
+ *       and stripping out characters that are not alphanumerics.
+ *       For example, "alpha-beta0gamma" becomes "AlphaBeta0gamma".
+=*/
+SCM
+ag_scm_string_to_camelcase(SCM str)
+{
+    int   len;
+    char* pzd, * res;
+    char* pzs;
+    bool cap_done = false;
+
+    if (! AG_SCM_STRING_P(str))
+        return SCM_UNDEFINED;
+
+    len = AG_SCM_STRLEN(str);
+    res = pzd = ag_scribble(len + 1);
+    pzs = (char*)(void*)AG_SCM_CHARS(str);
+
+    while (--len >= 0) {
+        unsigned int ch = *(pzs++);
+        if (IS_LOWER_CASE_CHAR(ch)) {
+            if (! cap_done) {
+                ch = toupper(ch);
+                cap_done = true;
+            }
+
+        } else if (IS_UPPER_CASE_CHAR(ch)) {
+            if (cap_done)
+                ch = tolower(ch);
+            else
+                cap_done = true;
+
+        } else if (IS_DEC_DIGIT_CHAR(ch)) {
+            cap_done = false;
+
+        } else {
+            cap_done = false;
+            continue;
+        }
+
+        *(pzd++) = ch;
+    }
+
+    return AG_SCM_STR2SCM(res, (pzd - res));
+}
 /*
  * Local Variables:
  * mode: C
