@@ -108,14 +108,6 @@ typedef struct {
     fmem_cookie_t * cookie;
 } cookie_fp_map_t;
 
-#ifndef AUTOGEN_BUILD
-/**
- * invent our own boolean rather than try to work around all the ways
- * different systems spell it.
- */
-typedef enum { AG_FALSE = 0, AG_TRUE = 1 } ag_bool;
-#endif
-
 static cookie_fp_map_t const * map    = NULL;
 static unsigned int            map_ct = 0;
 static unsigned int            map_alloc_ct = 0;
@@ -139,10 +131,10 @@ fmem_seek(void * cookie, seek_off_t offset, int dir);
 static int
 fmem_close(void * cookie);
 
-static ag_bool
+static bool
 fmem_config_user_buf(fmem_cookie_t * pFMC, void * buf, ssize_t len);
 
-static ag_bool
+static bool
 fmem_alloc_buf(fmem_cookie_t * pFMC, ssize_t len);
 /* = = = END-STATIC-FORWARD = = = */
 
@@ -415,7 +407,7 @@ fmem_close(void * cookie)
 /**
  * Configure the user supplied buffer.
  */
-static ag_bool
+static bool
 fmem_config_user_buf(fmem_cookie_t * pFMC, void * buf, ssize_t len)
 {
     /*
@@ -424,7 +416,7 @@ fmem_config_user_buf(fmem_cookie_t * pFMC, void * buf, ssize_t len)
     if (len == 0) {
         free(pFMC);
         errno = EINVAL;
-        return AG_FALSE;
+        return false;
     }
 
     pFMC->buffer = (buf_bytes_t*)buf;
@@ -452,7 +444,7 @@ fmem_config_user_buf(fmem_cookie_t * pFMC, void * buf, ssize_t len)
         buf_bytes_t *p = (buf_bytes_t*)buf;
 
         pFMC->eof = 0;
-        while ((*p != NUL) && (++(pFMC->eof) < len))  p++;
+        while ((*p != NUL) && (++(pFMC->eof) < (size_t)len))  p++;
         pFMC->next_ix =
             (pFMC->mode & FLAG_BIT(append)) ? pFMC->eof : 0;
     }
@@ -461,18 +453,18 @@ fmem_config_user_buf(fmem_cookie_t * pFMC, void * buf, ssize_t len)
      *  text mode - NUL terminate buffer, if it fits.
      */
     if (  ((pFMC->mode & FLAG_BIT(binary)) == 0)
-       && (pFMC->next_ix < len)) {
+       && (pFMC->next_ix < (size_t)len)) {
         pFMC->buffer[pFMC->next_ix] = NUL;
     }
 
     pFMC->buf_size = len;
-    return AG_TRUE;
+    return true;
 }
 
 /**
  * Allocate an initial buffer for fmem.
  */
-static ag_bool
+static bool
 fmem_alloc_buf(fmem_cookie_t * pFMC, ssize_t len)
 {
     /*
@@ -489,7 +481,7 @@ fmem_alloc_buf(fmem_cookie_t * pFMC, ssize_t len)
     if (pFMC->buffer == NULL) {
         errno = ENOMEM;
         free(pFMC);
-        return AG_FALSE;
+        return false;
     }
 
     /*
@@ -499,7 +491,7 @@ fmem_alloc_buf(fmem_cookie_t * pFMC, ssize_t len)
     pFMC->next_ix  = 0;
     pFMC->eof      = 0;
     pFMC->buf_size = len;
-    return AG_TRUE;
+    return true;
 }
 
 /*=export_func ag_fmemopen
@@ -698,7 +690,7 @@ ag_fmemioctl(FILE * fp, int req, ...)
     fmem_cookie_t * cookie;
     fmemc_get_buf_addr_t * gba;
 
-    if (req != IOCTL_FMEMC_GET_BUF_ADDR) {
+    if ((unsigned int)req != IOCTL_FMEMC_GET_BUF_ADDR) {
         /*
          *  It is not any of the IOCTL commands we know about.
          */
